library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity FT60x_axis_v1_0 is
	generic (
		-- Users to add parameters here

        C_USE_DBG	    : boolean := false;
        
        C_USE_GPIO	    : boolean := false;
        C_USE_SIWU	    : boolean := false;
        C_USE_WAKEUP    : boolean := false;
        C_USE_RESET	    : boolean := false;
        C_USE_TDEST	    : boolean := true;
        C_DEST_WIDTH	: integer range 1 to 2	:= 2;
        C_NUM_BYTES	    : integer range 2 to 4	:= 2   
	);
	port (
		-- Users to add ports here

        BE	      : inout std_logic_vector(C_NUM_BYTES-1 downto 0);
		DATA	  : inout std_logic_vector(C_NUM_BYTES*8-1 downto 0);
		SIWU_N	  : out std_logic;
		WAKEUP_N  : out std_logic;
		RESET_N	  : out std_logic;
		WR_N      : out std_logic;
		RD_N      : out std_logic;
		OE_N      : out std_logic;
		TXE_N     : in std_logic;
		RXF_N     : in std_logic;
		CLK  	  : in std_logic;
		
		-- Clock for AXI Streams..
		clk_axis	: out std_logic;
		
		dbg_ftdi_data	: out std_logic_vector(C_NUM_BYTES*8-1 downto 0);
		
		-- User ports ends
		-- Do not modify the ports beyond this line


		-- Ports of Axi Slave Bus Interface S_AXIS
		axis_aclk	: in std_logic;
		axis_aresetn	: in std_logic;

		--s_axis_aclk	: in std_logic;
        --s_axis_aresetn    : in std_logic;
		
		s_axis_tready	: out std_logic;
		s_axis_tdata	: in std_logic_vector(C_NUM_BYTES*8-1 downto 0);
		s_axis_tstrb	: in std_logic_vector(C_NUM_BYTES-1 downto 0);
		s_axis_tkeep	: in std_logic_vector(C_NUM_BYTES-1 downto 0);
		s_axis_tdest	: in std_logic_vector(C_DEST_WIDTH-1 downto 0);
		s_axis_tlast	: in std_logic;
		s_axis_tvalid	: in std_logic;

		-- Ports of Axi Master Bus Interface M_AXIS
		--m_axis_aclk	: in std_logic;
		--m_axis_aresetn	: in std_logic;
		m_axis_tvalid	: out std_logic;
		m_axis_tdata	: out std_logic_vector(C_NUM_BYTES*8-1 downto 0);
		m_axis_tstrb	: out std_logic_vector(C_NUM_BYTES-1 downto 0);
		m_axis_tkeep	: out std_logic_vector(C_NUM_BYTES-1 downto 0);
		m_axis_tdest	: out std_logic_vector(C_DEST_WIDTH-1 downto 0);
		m_axis_tlast	: out std_logic;
		m_axis_tready	: in std_logic
	);
end FT60x_axis_v1_0;

architecture arch_imp of FT60x_axis_v1_0 is

signal clk_i: std_logic;
signal data_i: std_logic_vector(C_NUM_BYTES*8-1 downto 0);
signal be_i: std_logic_vector(C_NUM_BYTES-1 downto 0);
signal dest_out_i: std_logic_vector(C_DEST_WIDTH-1 downto 0);
signal dest_in_i: std_logic_vector(C_DEST_WIDTH-1 downto 0);


begin
    clk_i <= CLK;
    clk_axis <= clk_i;
        
    data_i <= DATA;
    be_i <= BE;
    m_axis_tkeep <= be_i;
    m_axis_tdata <= data_i;
    
    -- must decode from DATA[15..8]
    m_axis_tdest <= dest_out_i;
    
    m_axis_tvalid <= '1';


    
    dbg_ftdi_data <= data_i; 

    WAKEUP_N <= '1';
    SIWU_N <= '1';
    RESET_N <= '1'; 
    

    OE_N <= '1';
    RD_N <= '1';
    WR_N <= '1';

end arch_imp;
